@@ -1,7 +1,7 @@
 /*
  * Alternate global C function prototypes for Mathomatic.
  *
- * Copyright (C) 1987-2011 George Gesslein II.
+ * Copyright (C) 1987-2012 George Gesslein II.
  */
 
 /* command function list */
@@ -17,7 +17,7 @@ int		sum_cmd(), product_cmd(), for_cmd(), integrate_cmd(), nintegrate_cmd(), lap
 
 /* various functions that don't return int */
 char		*dirname_win();
-char		*skip_space(), *skip_param();
+char		*skip_space(), *skip_comma_space(), *skip_param();
 char		*get_string();
 char		*parse_equation(), *parse_section(), *parse_var(), *parse_var2(), *parse_expr();
 char		*list_expression(), *list_equation(), *flist_equation_string();
