@@ -28,6 +28,40 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
 #include <termios.h>
 #endif
 
+/*
+ * Display the main Mathomatic startup message.
+ * fp is the output file stream pointer it goes to, and should be stdout or gfp,
+ * but really can go to any file you wish.
+ */
+void
+display_startup_message(fp)
+FILE	*fp;	/* output file pointer */
+{
+	long	es_size;
+
+#if	SECURE
+	fprintf(fp, _("Secure "));
+#else
+	if (security_level >= 2)
+		fprintf(fp, _("Secure "));
+	else if (security_level == -1)
+		fprintf(fp, "m4 ");
+#endif
+	fprintf(fp, "Mathomatic version %s\n", VERSION);
+	if (html_flag)
+		fprintf(fp, "Copyright &copy; 1987-2012 George Gesslein II.\n");
+	else
+		fprintf(fp, "Copyright (C) 1987-2012 George Gesslein II.\n");
+	es_size = (long) n_tokens * sizeof(token_type) * 2L / 1000L;
+	if (es_size >= 1000) {
+		fprintf(fp, _("%d equation spaces available in RAM; %ld megabytes per equation space.\n"),
+		    N_EQUATIONS, (es_size + 500L) / 1000L);
+	} else {
+		fprintf(fp, _("%d equation spaces available in RAM; %ld kilobytes per equation space.\n"),
+		    N_EQUATIONS, es_size);
+	}
+}
+
 /*
  * Standard function to report an error to the user.
  */
@@ -39,24 +73,43 @@ const char	*str;		/* constant string to display */
 #if	!SILENT && !LIBRARY
 	set_color(2);		/* set color to red */
 	printf("%s\n", str);
-	default_color();	/* restore to default color */
+	default_color(false);	/* restore to default color */
 #endif
 }
 
 /*
- * Standard function to report a warning to the user.
+ * Reset last call to error(), as if it didn't happen.
+ */
+void
+reset_error(void)
+{
+#if	!SILENT && !LIBRARY
+	if (error_str)
+		printf(_("Forgetting previous error.\n"));
+#endif
+	error_str = NULL;
+}
+
+/*
+ * Standard function to report a warning only once to the user.
  * A warning is less serious than an error.
  */
 void
 warning(str)
 const char	*str;		/* constant string to display */
 {
+	int	already_warned = false;
+
+	if (warning_str) {
+		if (strcmp(str, warning_str) == 0)
+			already_warned = true;
+	}
 	warning_str = str;	/* save reference to str, must be a constant string, temporary strings don't work */
 #if	!SILENT && !LIBRARY
-	if (debug_level >= 0) {
+	if (!already_warned && debug_level >= -1) {
 		set_color(1);		/* set color to yellow */
 		printf("Warning: %s\n", str);
-		default_color();	/* restore to default color */
+		default_color(false);	/* restore to default color */
 	}
 #endif
 }
@@ -214,9 +267,9 @@ init_mem(void)
 	return true;
 }
 
-#if	LIBRARY
+#if	LIBRARY || VALGRIND
 /*
- * Free the global expression storage arrays.
+ * Free the global expression storage arrays and other known memory buffers.
  * After calling this, memory usage is reset and Mathomatic becomes unusable,
  * so do not call unless finished with using the Mathomatic code.
  *
@@ -247,12 +300,23 @@ free_mem(void)
 		}
 	}
 	n_equations = 0;
+
+	for (i = 0; i < TEXT_ROWS; i++) {
+		if (vscreen[i]) {
+			free(vscreen[i]);
+			vscreen[i] = NULL;
+		}
+	}
+	current_columns = 0;
 }
 #endif
 
 #if	DEBUG
 /*
  * Use this function to check for any erroneous global conditions.
+ * Use of this function is rather paranoid, but helpful.
+ *
+ * Always returns true, or doesn't return on error.
  */
 int
 check_gvars(void)
@@ -281,6 +345,7 @@ check_gvars(void)
 /*
  * Initialize some important global variables to their defaults.
  * This is called on startup and by process() to reset the global flags to the default state.
+ * This is also called when processing is aborted with a longjmp(3).
  */
 void
 init_gvars(void)
@@ -498,15 +563,31 @@ alloc_next_espace(void)
 int
 next_espace(void)
 {
-	int	i;
+	int		i, j;
+	long		answer_v = 0;		/* Mathomatic answer variable */
 
 	i = alloc_next_espace();
 	if (i < 0) {
-		error(_("Out of free equation spaces."));
 #if	!SILENT
-		printf(_("Use the clear command on unnecessary equations and try again.\n"));
+		printf(_("Deleting old numeric calculations to free up equation spaces.\n"));
 #endif
-		longjmp(jmp_save, 3);	/* do not return */
+		parse_var(&answer_v, "answer");	/* convert to a Mathomatic variable */
+		for (j = 0; j < n_equations; j++) {
+			if (n_lhs[j] == 1 && lhs[j][0].kind == VARIABLE
+			    && lhs[j][0].token.variable == answer_v) {
+				/* delete calculation from memory */
+				n_lhs[j] = 0;
+				n_rhs[j] = 0;
+			}
+		}
+		i = alloc_next_espace();
+		if (i < 0) {
+			error(_("Out of free equation spaces."));
+#if	!SILENT
+			printf(_("Use the clear command on unnecessary equations and try again.\n"));
+#endif
+			longjmp(jmp_save, 3);	/* do not return */
+		}
 	}
 	return i;
 }
@@ -520,7 +601,9 @@ copy_espace(src, dest)
 int	src, dest;	/* equation space numbers */
 {
 	if (src == dest) {
-/*		error("Internal error: copy_espace() source and destination the same."); */
+#if	DEBUG
+		error_bug("Internal error: copy_espace() source and destination the same.");
+#endif
 		return;
 	}
 	blt(lhs[dest], lhs[src], n_lhs[src] * sizeof(token_type));
@@ -633,23 +716,21 @@ long	from_v, to_v;	/* Mathomatic variables */
 {
 	int	i;
 
-	if (en < 0 || en >= n_equations) {
+	if (empty_equation_space(en)) {
 #if	DEBUG
-		error_bug("Invalid equation number given to rename_var_in_es().");
+		error_bug("Invalid or empty equation number given to rename_var_in_es().");
 #else
 		return;
 #endif
 	}
-	if (n_lhs[en]) {
-		for (i = 0; i < n_lhs[en]; i += 2)
-			if (lhs[en][i].kind == VARIABLE
-			    && lhs[en][i].token.variable == from_v)
-				lhs[en][i].token.variable = to_v;
-		for (i = 0; i < n_rhs[en]; i += 2)
-			if (rhs[en][i].kind == VARIABLE
-			    && rhs[en][i].token.variable == from_v)
-				rhs[en][i].token.variable = to_v;
-	}
+	for (i = 0; i < n_lhs[en]; i += 2)
+		if (lhs[en][i].kind == VARIABLE
+		    && lhs[en][i].token.variable == from_v)
+			lhs[en][i].token.variable = to_v;
+	for (i = 0; i < n_rhs[en]; i += 2)
+		if (rhs[en][i].kind == VARIABLE
+		    && rhs[en][i].token.variable == from_v)
+			rhs[en][i].token.variable = to_v;
 }
 
 /*
@@ -677,8 +758,10 @@ long		v;		/* variable to substitute with expression */
 			if (*np + len - 1 > n_tokens) {
 				error_huge();
 			}
-			blt(&equation[j+len], &equation[j+1], (*np - (j + 1)) * sizeof(token_type));
-			*np += len - 1;
+			if (len > 1) {
+				blt(&equation[j+len], &equation[j+1], (*np - (j + 1)) * sizeof(token_type));
+				*np += len - 1;
+			}
 			blt(&equation[j], expression, len * sizeof(token_type));
 			for (k = j; k < j + len; k++)
 				equation[k].level += level;
@@ -828,7 +911,11 @@ int	i;	/* equation space number */
 		return true;
 	}
 	if (n_lhs[i] <= 0) {
-		error(_("Equation space is empty."));
+		if (i == cur_equation) {
+			error(_("Current equation space is empty."));
+		} else {
+			error(_("Equation space is empty."));
+		}
 		return true;
 	}
 	return false;
@@ -848,7 +935,6 @@ current_not_defined(void)
 	if (i < 0 || i >= n_equations) {
 		error(_("Current equation number out of range; reset to 1."));
 		i = cur_equation = 0;
-		return true;
 	}
 	if (n_lhs[i] <= 0) {
 		error(_("No current equation or expression."));
@@ -874,7 +960,7 @@ int	n;		/* maximum size of "string" in bytes */
 	return NULL;
 #else
 	int	i;
-#if	READLINE
+#if	READLINE || EDITLINE
 	char	*cp;
 #endif
 
@@ -886,7 +972,8 @@ int	n;		/* maximum size of "string" in bytes */
 		prompt_str[0] = '\0';	/* don't display a prompt */
 	}
 	input_column = strlen(prompt_str);
-#if	READLINE
+	fflush(NULL);	/* flush everything before gathering input */
+#if	READLINE || EDITLINE
 	if (readline_enabled) {
 		cp = readline(prompt_str);
 		if (cp == NULL) {
@@ -919,6 +1006,10 @@ int	n;		/* maximum size of "string" in bytes */
 		exit_program(0);
 	}
 #endif
+	if (abort_flag) {
+		abort_flag = false;
+		longjmp(jmp_save, 13);
+	}
 	/* Fix an fgets() peculiarity: */
 	i = strlen(string) - 1;
 	if (i >= 0 && string[i] == '\n') {
@@ -947,9 +1038,11 @@ get_yes_no(void)
 	char	*cp;
 	char	buf[MAX_CMD_LEN];
 
+#if	0
 	if (!isatty(0)) {
 		return false;
 	}
+#endif
 	for (;;) {
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
@@ -1030,7 +1123,7 @@ char	*cp;
 /*
  * Process an equation number range given in text string "*cpp".
  * Skip past all spaces and update "*cpp" to point to the next argument if successful.
- * If no equation number or range is given, assume the current equation is wanted and
+ * If no equation number or range is given, or it is invalid, assume the current equation is wanted and
  * don't skip anything.
  *
  * Return true if successful,
@@ -1044,8 +1137,9 @@ int	*ip, *jp;
 {
 	int	i;
 	char	*cp;
+	int	rv;
 
-	cp = skip_space(*cpp);
+	cp = skip_comma_space(*cpp);
 	if (is_all(cp)) {
 		cp = skip_param(cp);
 		*ip = 0;
@@ -1053,30 +1147,45 @@ int	*ip, *jp;
 		while (*jp > 0 && n_lhs[*jp] == 0)
 			(*jp)--;
 	} else {
+		if (*cp == '0') {
+			goto use_current;
+		}
 		if (isdigit(*cp)) {
 			*ip = strtol(cp, &cp, 10) - 1;
 		} else {
 			*ip = cur_equation;
 		}
 		if (*cp != '-') {
-			if (*cp == '\0' || isspace(*cp)) {
+			if (*cp == '\0' || *cp == ',' || isspace(*cp)) {
 				if (not_defined(*ip)) {
 					return false;
 				}
 				*jp = *ip;
-				*cpp = skip_space(cp);
+				*cpp = skip_comma_space(cp);
 				return true;
 			} else {
 use_current:
 				*jp = *ip = cur_equation;
 #if	1
-				return(!empty_equation_space(cur_equation));	/* don't display error message */
+				rv = !empty_equation_space(cur_equation);	/* don't display error message */
+				if (rv) {
+					debug_string(1, _("Defaulting to the current equation space."));
+				} else {
+					debug_string(1, _("Defaulting to current empty equation space."));
+				}
 #else
-				return(!current_not_defined());		/* display an error message if error */
+				rv = !current_not_defined();		/* display an error message if error */
+				if (rv) {
+					debug_string(1, _("Defaulting to the current equation space."));
+				}
 #endif
+				return rv;
 			}
 		}
 		(cp)++;
+		if (*cp == '0') {
+			goto use_current;
+		}
 		if (isdigit(*cp)) {
 			*jp = strtol(cp, &cp, 10) - 1;
 		} else {
@@ -1095,14 +1204,14 @@ use_current:
 			*jp = i;
 		}
 	}
-	cp = skip_space(cp);
+	cp = skip_comma_space(cp);
 	for (i = *ip; i <= *jp; i++) {
 		if (n_lhs[i] > 0) {
 			*cpp = cp;
 			return true;
 		}
 	}
-	error(_("No equations defined in specified range."));
+	error(_("No expressions defined in specified range."));
 	return false;
 }
 
@@ -1118,7 +1227,7 @@ extra_characters(cp)
 char	*cp;	/* command line string */
 {
 	if (cp) {
-		cp = skip_space(cp);
+		cp = skip_comma_space(cp);
 		if (*cp) {
 			printf(_("\nError: \"%s\" not required on input line.\n"), cp);
 			error(_("Extra characters or unrecognized argument."));
@@ -1161,22 +1270,23 @@ char	*cp;	/* character pointer */
 }
 
 /*
- * Skip over comma and space characters.
+ * Skip over a possible comma and space characters.
  */
 char *
 skip_comma_space(cp)
 char	*cp;	/* character pointer */
 {
 	if (cp) {
-		while (*cp && (*cp == ',' || isspace(*cp)))
-			cp++;
+		cp = skip_space(cp);
+		if (*cp == ',')
+			cp = skip_space(cp + 1);
 	}
 	return cp;
 }
 
 /*
  * Enhanced decimal strtol().
- * Skips trailing spaces.
+ * Skips trailing spaces or commas.
  */
 long
 decstrtol(cp, cpp)
@@ -1185,8 +1295,8 @@ char	*cp, **cpp;
 	long	l;
 
 	l = strtol(cp, cpp, 10);
-	if (cpp && cp != *cpp) {
-		*cpp = skip_space(*cpp);
+	if (cpp && *cpp && cp != *cpp) {
+		*cpp = skip_comma_space(*cpp);
 	}
 	return l;
 }
@@ -1203,7 +1313,7 @@ int	ch;
 
 /*
  * Skip over the current parameter in a Mathomatic command line string.
- * Parameters are usually separated with spaces.
+ * Parameters are usually separated with spaces or a comma or equals sign.
  *
  * Returns a string (character pointer) to the next parameter.
  */
@@ -1402,7 +1512,6 @@ int		n1;	/* expression length */
 	int	i;
 	int	level;
 
-#if	1
 	for (i = n1 - 2; i > 2; i -= 2) {
 		if (p1[i].token.operatr != POWER)
 			continue;
@@ -1414,16 +1523,6 @@ int		n1;	/* expression length */
 			}
 		}
 	}
-#else
-	for (i = n1 - 2; i > 0; i -= 2) {
-		if (p1[i].token.operatr != POWER)
-			continue;
-		level = p1[i].level;
-		if (p1[i+1].level == level && p1[i+1].kind == CONSTANT && fmod(p1[i+1].token.constant, 1.0) != 0.0) {
-			return true;
-		}
-	}
-#endif
 	return false;
 }
 
@@ -1442,3 +1541,117 @@ double	denominator;
 	}
 	return false;
 }
+
+#if	CYGWIN || MINGW
+/*
+ * dirname(3) function for Microsoft Windows.
+ * dirname(3) strips the non-directory suffix from a filename.
+ */
+char *
+dirname_win(cp)
+char	*cp;	/* string containing filename to modify */
+{
+	int	i;
+
+	if (cp == NULL)
+		return(".");
+	i = strlen(cp);
+	while (i >= 0 && cp[i] != '\\' && cp[i] != '/')
+		i--;
+	if (i < 0)
+		return(".");
+	cp[i] = '\0';
+	return(cp);
+}
+#endif
+
+#if	!SECURE
+/*
+ * Load set options from startup file "~/.mathomaticrc".
+ *
+ * Return false if there was an error reading the startup file,
+ * otherwise return true.
+ */
+int
+load_rc(return_true_if_no_file, ofp)
+int	return_true_if_no_file;
+FILE	*ofp;	/* if non-NULL, display each line as read in to this file */
+{
+	FILE	*fp = NULL;
+	char	buf[MAX_CMD_LEN];
+	char	*cp;
+	int	rv = true;
+
+	cp = getenv("HOME");
+	if (cp) {
+		snprintf(rc_file, sizeof(rc_file), "%s/%s", cp, ".mathomaticrc");
+		fp = fopen(rc_file, "r");
+	}
+#if	CYGWIN || MINGW
+	if (fp == NULL && cp) {
+		snprintf(rc_file, sizeof(rc_file), "%s/%s", cp, "mathomatic.rc");
+		fp = fopen(rc_file, "r");
+	}
+	if (fp == NULL && dir_path) {
+		snprintf(rc_file, sizeof(rc_file), "%s/%s", dir_path, "mathomatic.rc");
+		fp = fopen(rc_file, "r");
+	}
+#endif
+	if (fp == NULL) {
+		if (return_true_if_no_file) {
+			return true;
+		} else {
+			perror(rc_file);
+			return false;
+		}
+	}
+	if (!quiet_mode && !eoption) {
+		printf(_("Loading startup set options from \"%s\".\n"), rc_file);
+	}
+	while ((cp = fgets(buf, sizeof(buf), fp)) != NULL) {
+		if (ofp)
+			fprintf(ofp, "%s", cp);
+		set_error_level(cp);
+		if (!set_options(cp, true))
+			rv = false;
+	}
+	if (fclose(fp)) {
+		rv = false;
+		perror(rc_file);
+	}
+	return rv;
+}
+
+#if	0	/* not currently used */
+/*
+ * Display set options from startup file "~/.mathomaticrc".
+ *
+ * Return false if there was an error reading the startup file,
+ * otherwise return true.
+ */
+int
+display_rc(ofp)
+FILE	*ofp;
+{
+	FILE	*fp = NULL;
+	char	buf[MAX_CMD_LEN];
+	char	*cp;
+	int	rv = true;
+
+	printf(_("Displaying startup set options from \"%s\":\n\n"), rc_file);
+	fp = fopen(rc_file, "r");
+	if (fp == NULL) {
+		perror(rc_file);
+		return false;
+	}
+	while ((cp = fgets(buf, sizeof(buf), fp)) != NULL) {
+		fprintf(ofp, "%s", cp);
+	}
+	if (fclose(fp)) {
+		rv = false;
+		perror(rc_file);
+	}
+	return rv;
+}
+#endif
+#endif
