@@ -59,10 +59,11 @@ extern volatile int	screen_rows;
 extern int		finance_option;
 extern int		autosolve;
 extern int		autocalc;
+extern int		autodelete;
 extern int		autoselect;
 extern char		special_variable_characters[256];
 extern char		plot_prefix[256];
-extern int		integer_coefficients;
+extern int		factor_out_all_numeric_gcds;
 extern int		right_associative_power;
 extern int		power_starstar;
 #if	!SILENT
@@ -71,6 +72,7 @@ extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
 extern int		bold_colors;
+extern int		text_color;
 extern int		cur_color;
 extern int		html_flag;
 extern int		readline_enabled;
@@ -87,15 +89,17 @@ extern char		*prog_name;
 extern char		*var_names[MAX_VAR_NAMES];
 extern char		var_str[MAX_VAR_LEN+80];
 extern char		prompt_str[MAX_PROMPT_LEN];
-#if	!SECURE && !LIBRARY
+#if	!SECURE
 extern char		rc_file[MAX_CMD_LEN];
 #endif
 
 #if	CYGWIN || MINGW
 extern char		*dir_path;
 #endif
-#if	READLINE
+#if	READLINE || EDITLINE
 extern char		*last_history_string;
+#endif
+#if	READLINE
 extern char		*history_filename;
 extern char		history_filename_storage[MAX_CMD_LEN];
 #endif
@@ -104,10 +108,12 @@ extern double		unique[];
 extern int		ucnt[];
 extern int		uno;
 
+extern int		previous_return_value;
 extern sign_array_type	sign_array;
 extern FILE		*default_out;
 extern FILE		*gfp;
 extern char		*gfp_filename;
+extern int		gfp_append_flag;
 extern jmp_buf		jmp_save;
 extern int		eoption;
 extern int		test_mode;
