@@ -53,7 +53,8 @@ double	value;
 
 	uno = 0;
 	nn = value;
-	if (nn == 0.0) {
+	if (nn == 0.0 || !isfinite(nn)) {
+		/* zero or not finite */
 		return false;
 	}
 	if (fabs(nn) >= MAX_K_INTEGER) {
@@ -85,8 +86,12 @@ double	value;
 	if (uno == 0) {
 		try_factor(1.0);
 	}
-	if (nn != 1.0 || value != multiply_out_unique()) {
-		error_bug("Internal error factoring integers.");
+/* Do some floating point arithmetic self-checking.  If the following fails, it is due to a floating point bug. */
+	if (nn != 1.0) {
+		error_bug("Internal error factoring integers (final nn != 1.0).");
+	}
+	if (value != multiply_out_unique()) {
+		error_bug("Internal error factoring integers (result array value is incorrect).");
 	}
 	return true;
 }
@@ -99,6 +104,11 @@ static void
 try_factor(arg)
 double	arg;
 {
+#if	DEBUG
+	if (fmod(arg, 1.0) != 0.0) {
+		error_bug("Trying factor that is not an integer!");
+	}
+#endif
 	while (fmod(nn, arg) == 0.0) {
 		if (uno > 0 && ucnt[uno-1] > 0 && unique[uno-1] == arg) {
 			ucnt[uno-1]++;
@@ -109,9 +119,11 @@ double	arg;
 			ucnt[uno++] = 1;
 		}
 		nn /= arg;
+#if	DEBUG
 		if (fmod(nn, 1.0) != 0.0) {
-			error_bug("Turned non-integer in try_factor().");
+			error_bug("nn turned non-integer in try_factor().");
 		}
+#endif
 		sqrt_value = 1.0 + sqrt(fabs(nn));
 		if (fabs(nn) <= 1.5 || fabs(arg) <= 1.5)
 			break;
@@ -131,6 +143,11 @@ multiply_out_unique(void)
 
 	d = 1.0;
 	for (i = 0; i < uno; i++) {
+#if	DEBUG
+		if (ucnt[i] < 0) {
+			error_bug("Error in ucnt[] being negative.");
+		}
+#endif
 		for (j = 0; j < ucnt[i]; j++) {
 			d *= unique[i];
 		}
@@ -139,29 +156,41 @@ multiply_out_unique(void)
 }
 
 /*
- * Display the integer prime factors in the unique[] array.
+ * Display the integer prime factors in the unique[] array, even if mangled.
  * Must have had a successful call to factor_one(value) previously,
  * to fill out unique[] with the prime factors of value.
+ *
+ * Return true if successful.
  */
-void
+int
 display_unique(void)
 {
 	int	i;
 	double	value;
 
+	if (uno <= 0)
+		return false;
 	value = multiply_out_unique();
 	fprintf(gfp, "%.0f = ", value);
 	for (i = 0; i < uno;) {
-		fprintf(gfp, "%.0f", unique[i]);
+		if (ucnt[i] > 0) {
+			fprintf(gfp, "%.0f", unique[i]);
+		} else {
+			i++;
+			continue;
+		}
 		if (ucnt[i] > 1) {
 			fprintf(gfp, "^%d", ucnt[i]);
 		}
-		i++;
+		do {
+			i++;
+		} while (i < uno && ucnt[i] <= 0);
 		if (i < uno) {
 			fprintf(gfp, " * ");
 		}
 	}
 	fprintf(gfp, "\n");
+	return true;
 }
 
 /*
@@ -174,6 +203,12 @@ is_prime(void)
 {
 	double	value;
 
+	if (uno <= 0) {
+#if	DEBUG
+		error_bug("uno == 0 in is_prime().");
+#endif
+		return false;
+	}
 	value = multiply_out_unique();
 	if (value < 2.0)
 		return false;
@@ -255,15 +290,22 @@ int		*np;
 
 /*
  * Factor integers in an equation space.
+ *
+ * Return true if something was factored.
  */
-void
+int
 factor_int_equation(n)
 int	n;	/* equation space number */
 {
+	int	rv = false;
+
 	if (empty_equation_space(n))
-		return;
-	factor_int(lhs[n], &n_lhs[n]);
-	factor_int(rhs[n], &n_rhs[n]);
+		return rv;
+	if (factor_int(lhs[n], &n_lhs[n]))
+		rv = true;
+	if (factor_int(rhs[n], &n_rhs[n]))
+		rv = true;
+	return rv;
 }
 
 /*
@@ -345,7 +387,7 @@ int		level_code;
 	if (modified)
 		return true;
 	improve_readability = ((level_code & 3) > 1 || ((level_code & 3) && (level == 1)));
-	gcd_flag = ((improve_readability && integer_coefficients) || (level_code & 4));
+	gcd_flag = ((improve_readability && factor_out_all_numeric_gcds) || (level_code & 4));
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level == level) {
 			switch (equation[i].kind) {
@@ -458,16 +500,18 @@ int		level_code;
 					d = equation[i].token.constant;
 				}
 			}
-#if	1
+#if	0	/* was 1; changed to 0 for optimal results and so 180*(sides-2) simplification works nicely. */
 			if (!gcd_flag && minimum >= 1.0) {
 				minimum = 1.0;
 				break;
 			}
 #endif
+#if	1
 			if ((minimum < 1.0) && (fmod(d, 1.0) == 0.0)) {
 				minimum = 1.0;
 				break;
 			}
+#endif
 /* Make sure division by the number to factor out results in an integer: */
 			if (fmod(d, minimum) != 0.0) {
 				minimum = 1.0;	/* result not an integer */
