@@ -42,8 +42,9 @@ int		n_lhs[N_EQUATIONS],		/* number of tokens in each lhs[], 0 means equation sp
 
 token_type	*tlhs,				/* LHS during solve and temporary storage for expressions, quotient for poly_div() and smart_div(). */
 		*trhs,				/* RHS during solve and temporary storage for expressions, remainder for poly_div() and smart_div(). */
-		*tes,				/* temporary equation side, used in commands, simpa_repeat_side(), simple_frac_repeat_side(), etc. */
-		*scratch;			/* very temporary storage for expressions, used only in low level routines for expression manipulation. */
+		*tes,				/* Temporary Equation Side, used in commands, simpa_repeat_side(), simple_frac_repeat_side(), etc. */
+		*scratch;			/* Very temporary storage for expressions, used only in low level routines for expression manipulation. */
+						/* Do not run any functions on scratch[], except for blt() (which is memmove(3)). */
 
 int		n_tlhs,				/* number of tokens in tlhs */
 		n_trhs,				/* number of tokens in trhs */
@@ -67,17 +68,22 @@ int		rationalize_denominators = true;	/* try to rationalize denominators if true
 int		modulus_mode = 2;				/* true for mathematically correct modulus */
 volatile int	screen_columns = STANDARD_SCREEN_COLUMNS;	/* screen width of the terminal; 0 = infinite */
 volatile int	screen_rows = STANDARD_SCREEN_ROWS;		/* screen height of the terminal; 0 = infinite */
-int		finance_option;				/* for displaying dollars and cents */
+int		finance_option = -1;				/* for displaying dollars and cents */
 int		autosolve = true;			/* Allows solving by typing the variable name at the main prompt */
 int		autocalc = true;			/* Allows automatically calculating a numerical expression */
+int		autodelete = false;			/* Automatically deletes the previous calculated numerical expression when a new one is entered */
 int		autoselect = true;			/* Allows selecting equation spaces by typing the number */
 #if	LIBRARY
 char		special_variable_characters[256] = "\\[]";	/* allow backslash in variable names for Latex compatibility */
 #else
 char		special_variable_characters[256] = "'\\[]";	/* user defined characters for variable names, '\0' terminated */
 #endif
+#if	MINGW
+char		plot_prefix[256] = "set grid; set xlabel 'X'; set ylabel 'Y';";		/* prefix fed into gnuplot before the plot command */
+#else
 char		plot_prefix[256] = "set grid; set xlabel \"X\"; set ylabel \"Y\";";	/* prefix fed into gnuplot before the plot command */
-int		integer_coefficients = false;		/* if true, factor out the GCD of rational coefficients */
+#endif
+int		factor_out_all_numeric_gcds = false;	/* if true, factor out the GCD of rational coefficients */
 int		right_associative_power;		/* if true, evaluate power operators right to left */
 int		power_starstar;				/* if true, display power operator as "**", otherwise "^" */
 #if	!SILENT
@@ -86,17 +92,18 @@ int		debug_level;				/* current debug level */
 
 /* variables having to do with color output mode */
 #if	LIBRARY || NO_COLOR
-int		color_flag = false;		/* library shouldn't default to color mode */
+int		color_flag = 0;			/* library shouldn't default to color mode */
 #else
-int		color_flag = true;		/* "set color" flag, true for color output mode */
+int		color_flag = 1;			/* "set color" flag; 0 for no color, 1 for color, 2 for alternative color output mode */
 #endif
 #if	BOLD_COLOR
-int		bold_colors = true;		/* "set bold color" flag for brighter colors */
+int		bold_colors = 1;		/* "set bold color" flag for brighter colors */
 #else
-int		bold_colors = false;
+int		bold_colors = 0;		/* bold_colors must be 0 or 1; 0 is dim */
 #endif
+int		text_color = -1;		/* Current normal text color, -1 for no color */
 int		cur_color = -1;			/* memory of current color on the terminal */
-int		html_flag;			/* true for HTML output mode, color mode may apply */
+int		html_flag;			/* 1 for HTML mode on all standard output; 2 for HTML mode on all output, even redirected output */
 
 /* double precision floating point epsilon constants for number comparisons for equivalency */
 double		small_epsilon	= 0.000000000000005;	/* for ignoring small, floating point round-off errors */
@@ -107,15 +114,17 @@ char		*prog_name = "mathomatic";	/* name of this program */
 char		*var_names[MAX_VAR_NAMES];	/* index for storage of variable name strings */
 char		var_str[MAX_VAR_LEN+80];	/* temp storage for listing a variable name */
 char		prompt_str[MAX_PROMPT_LEN];	/* temp storage for the prompt string */
-#if	!SECURE && !LIBRARY
+#if	!SECURE
 char		rc_file[MAX_CMD_LEN];		/* pathname for the set options startup file */
 #endif
 
 #if	CYGWIN || MINGW
 char		*dir_path;			/* directory path to the executable */
 #endif
-#if	READLINE
+#if	READLINE || EDITLINE
 char		*last_history_string;		/* To prevent repeated, identical entries.  Must not point to temporary string. */
+#endif
+#if	READLINE
 char		*history_filename;
 char		history_filename_storage[MAX_CMD_LEN];
 #endif
@@ -126,14 +135,16 @@ int		ucnt[64];		/* number of times the factor occurs */
 int		uno;			/* number of unique factors stored in unique[] */
 
 /* misc. variables */
+int		previous_return_value = 1;	/* Return value of last command entered. */
 sign_array_type	sign_array;		/* for keeping track of unique "sign" variables */
 FILE		*default_out;		/* file pointer where all gfp output goes by default */
 FILE		*gfp;			/* global output file pointer, for dynamically redirecting Mathomatic output */
 char		*gfp_filename;		/* filename associated with gfp if redirection is happening */
+int		gfp_append_flag;	/* true if appending to gfp, false if overwriting */
 jmp_buf		jmp_save;		/* for setjmp(3) to longjmp(3) to when an error happens deep within this code */
 int		eoption;		/* -e option flag */
 int		test_mode;		/* test mode flag (-t) */
-int		demo_mode;		/* demo mode flag (-d), don't load rc file when true */
+int		demo_mode;		/* demo mode flag (-d), don't load rc file or pause commands when true */
 int		quiet_mode;		/* quiet mode (-q, don't display prompts) */
 int		echo_input;		/* if true, echo input */
 int		readline_enabled = true;	/* set to false (-r) to disable readline */
@@ -160,6 +171,6 @@ int		result_en = -1;		/* equation number of the returned result, if stored in an
 const char	*error_str;		/* last error string */
 const char	*warning_str;		/* last warning string */
 
-/* Screen character array, for buffering page-at-a-time string output: */
+/* Screen character array, for buffering page-at-a-time 2D string output: */
 char		*vscreen[TEXT_ROWS];
 int		current_columns;
