@@ -47,7 +47,8 @@ static short windows_carray[] = {
 }; 
 
 extern HANDLE hOut;
-#else
+#endif
+
 /* ANSI terminal color code array for 8 color ANSI; we don't use black or white */
 /* because the background may be the same color, so there are only 6 colors here. */
 static int	carray[] = {
@@ -58,7 +59,6 @@ static int	carray[] = {
 	35,	/* magenta */
 	36,	/* cyan */
 };
-#endif
 
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
 #define MODULUS_STRING	" % "	/* string displayed for the modulus operator */
@@ -100,20 +100,26 @@ static char	*html_carray[] = {
 void
 reset_attr(void)
 {
-FILE	*fp;
+	FILE	*fp;
 
-#if	LIBRARY
-	fp = gfp;
-#else
-	fp = stdout;
-#endif
+	if (html_flag == 2) {
+		fp = gfp;
+	} else {
+		fp = stdout;
+	}
+#if	!LIBRARY
 	fflush(NULL);	/* flush all output */
+#endif
 	if (color_flag && cur_color >= 0) {
 		if (html_flag) {
 			fprintf(fp, "</font>");
 		} else {
 #if	WIN32_CONSOLE_COLORS
-                        SetConsoleTextAttribute(hOut, FOREGROUND_WHITE);
+			if (color_flag == 2) {
+	                        fprintf(fp, "\033[0m");
+			} else {
+	                        SetConsoleTextAttribute(hOut, FOREGROUND_WHITE);
+			}
 #else
                         fprintf(fp, "\033[0m");
 #endif
@@ -137,10 +143,9 @@ int	color;
 {
 	int	rv = -1;
 
-#if	!LIBRARY
-	if (gfp != stdout)
+	if (html_flag != 2 && gfp != stdout) {
 		return rv;
-#endif
+	}
 	if (color_flag) {
 		if (cur_color == color)	/* color already set */
 			return rv;
@@ -155,10 +160,14 @@ int	color;
 			}
 		} else {
 #if	WIN32_CONSOLE_COLORS
-                        short attr = windows_carray[rv = (color % ARR_CNT(windows_carray))];
-                        if (bold_colors)
-                                 attr |= FOREGROUND_INTENSITY;
-                        SetConsoleTextAttribute(hOut, attr);
+			if (color_flag == 2) {
+	                        fprintf(gfp, "\033[%d;%dm", bold_colors, carray[rv = (color % ARR_CNT(carray))]);
+			} else {
+	                        short attr = windows_carray[rv = (color % ARR_CNT(windows_carray))];
+        	                if (bold_colors)
+                	                 attr |= FOREGROUND_INTENSITY;
+				SetConsoleTextAttribute(hOut, attr);
+			}
 #else
                         fprintf(gfp, "\033[%d;%dm", bold_colors, carray[rv = (color % ARR_CNT(carray))]);
 #endif
@@ -172,25 +181,31 @@ int	color;
  * Set normal text color for subsequent output.
  */
 void
-default_color(void)
+default_color(set_no_color_flag)
+int	set_no_color_flag;	/* If true, set no color even if text_color is set.  Normally this would be false. */
 {
-#if	!LIBRARY
-	if (gfp != stdout) {
+	if (html_flag != 2 && gfp != stdout) {
 		return;
 	}
-#endif
 	if (color_flag && cur_color >= 0) {
 		if (html_flag) {
 			fprintf(gfp, "</font>");
 		} else {
 #if	WIN32_CONSOLE_COLORS
-			SetConsoleTextAttribute(hOut, FOREGROUND_WHITE);
+			if (color_flag == 2) {
+				fprintf(gfp, "\033[0m");
+			} else {
+				SetConsoleTextAttribute(hOut, FOREGROUND_WHITE);
+			}
 #else
 			fprintf(gfp, "\033[0m");
 #endif
 		}
 	}
 	cur_color = -1;
+	if (text_color >= 0 && !set_no_color_flag) {
+		set_color(text_color);
+	}
 	fflush(gfp);
 }
 
@@ -205,14 +220,17 @@ display_all_colors(void)
 	int	i, j;
 
 	i = 0;
-	if (set_color(i) < 0)
+	default_color(true);
+	if (set_color(i) < 0) {
+		default_color(false);
 		return false;
+	}
 	do {
 		printf("#");
 		i++;
 		j = set_color(i);
 	} while (j > 0);
-	default_color();
+	default_color(false);
 	return(j >= 0);
 }
 
@@ -264,7 +282,7 @@ int	export_flag;	/* non-zero for exportable format (readable by other math progr
 	if (export_flag == 1) {
 		len += fprintf(gfp, ";");
 	}
-#if	CYGWIN || MINGW
+#if	CYGWIN
 	fprintf(gfp, "\r\n");	/* might be redirecting to a Microsoft text file */
 #else
 	fprintf(gfp, "\n");
@@ -307,7 +325,9 @@ token_type	*p2;
 int		n2;
 {
 	if (debug_level >= level) {
-		fprintf(gfp, _("level %d: "), level);
+		if (level >= -2) {
+			fprintf(gfp, _("level %d: "), level);
+		}
 		list_proc(p1, n1, false);
 		if (p2 && n2 > 0) {
 			fprintf(gfp, EQUATE_STRING);
@@ -651,7 +671,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
 				trim_zeros(buf);
 			} else if (export_flag || high_prec) {
 				snprintf(buf, sizeof(buf), "%.*g", export_precision, p1[i].token.constant);
-			} else if (finance_option) {
+			} else if (finance_option >= 0) {
 #if	THOUSANDS_SEPARATOR	/* Fails miserably in MinGW and possibly others, displaying nothing but the format string. */
 				snprintf(buf, sizeof(buf), "%'.*f", finance_option, p1[i].token.constant);
 #else
@@ -722,13 +742,15 @@ int		export_flag;	/* flag for exportable format (usually false) */
 			set_color(j);
 	}
 	if (cflag)
-		default_color();
+		default_color(false);
 	return len;
 }
 
 /*
- * Return true if expression is a valid integer expression for
+ * Return 1 (true) or -1 if expression is a valid integer expression for
  * list_code().
+ * Return 0 if it is definitely a non-integer expression.
+ * Return -1 if it contains non-integer divide operators, but is OK otherwise.
  */
 int
 int_expr(p1, n)
@@ -736,24 +758,27 @@ token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 {
 	int	i;
+	int	rv = 1;
 
 	for (i = 0; i < n; i++) {
 		switch (p1[i].kind) {
 		case CONSTANT:
 			if (fmod(p1[i].token.constant, 1.0) != 0.0) {
-				return false;
+				return 0;
 			}
 			break;
 		case VARIABLE:
 			if (p1[i].token.variable < IMAGINARY) {
-				return false;
+				return 0;
 			}
 			break;
 		case OPERATOR:
+			if (p1[i].token.operatr == DIVIDE)
+				rv = -1;
 			break;
 		}
 	}
-	return true;
+	return rv;
 }
 
 /*
@@ -895,7 +920,7 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 							}
 							break;
 						case FACTORIAL:
-							APPEND("fact");
+							APPEND("factorial");
 							break;
 						}
 						break;
@@ -1125,7 +1150,7 @@ make_smaller:
 	if (screen_columns && use_screen_columns && width >= screen_columns) {
 		/* output too wide to fit screen, output in single-line format */
 		width = list1_sub(n, false);
-#if	CYGWIN || MINGW
+#if	CYGWIN
 		fprintf(gfp, "\r\n");	/* Be consistent with list1_sub() output. */
 #else
 		fprintf(gfp, "\n");
@@ -1187,7 +1212,7 @@ int		*highp, *lowp;
 
 	rv = flist_recurse(p1, n, out_flag, string, sbuffer_size, 0, pos, 1, highp, lowp);
 	if (out_flag && (string == NULL)) {
-		default_color();
+		default_color(false);
 	}
 	return rv;
 }
@@ -1229,11 +1254,7 @@ int		*highp, *lowp;
 	}
 	oflag = (out_flag && line == cur_line);
 	cflag = (oflag && string == NULL);
-#if	LIBRARY
-	html_out = html_flag;
-#else
-	html_out = (html_flag && gfp == stdout);
-#endif
+	html_out = ((html_flag == 2) || (html_flag && gfp == stdout));
 	if (oflag) {
 		for (; cur_pos < pos; cur_pos++) {
 			APPEND2(" ");
@@ -1380,7 +1401,7 @@ check_again:
 			    && (i + 1) < n && p1[i].level == p1[i+1].level && p1[i+1].token.operatr == TIMES) {
 				i++;
 				len += snprintf(buf, sizeof(buf), "-");
-			} else if (finance_option) {
+			} else if (finance_option >= 0) {
 				if (p1[i].token.constant < 0.0) {
 #if	THOUSANDS_SEPARATOR
 					len += snprintf(buf, sizeof(buf), "(%'.*f)", finance_option, p1[i].token.constant);
