@@ -59,6 +59,12 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  *
  * See the file "am.h" to start understanding the Mathomatic code and
  * to adjust memory usage.
+ *
+ * C types "long long" and "long double" are not used at all in Mathomatic,
+ * because many architectures and compilers do not support them.  These large C
+ * types can be used in the Mathomatic Prime Number Tools though, producing
+ * larger prime numbers and quickly on a 64-bit computer.  64 bits is the size
+ * of a double, so Mathomatic runs optimally on a 64-bit system.
  */
 
 #if	!LIBRARY	/* This comments out this whole file if compiling as the symbolic math library. */
@@ -75,27 +81,6 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
 HANDLE	hOut;
 #endif
 
-void
-display_startup_message(fp)
-FILE	*fp;
-{
-#if	SECURE
-	fprintf(fp, _("Secure "));
-#else
-	if (security_level >= 2)
-		fprintf(fp, _("Secure "));
-	else if (security_level == -1)
-		fprintf(fp, "m4 ");
-#endif
-	fprintf(fp, "Mathomatic version %s\n", VERSION);
-	if (html_flag)
-		fprintf(fp, "Copyright &copy; 1987-2012 George Gesslein II.\n");
-	else
-		fprintf(fp, "Copyright (C) 1987-2012 George Gesslein II.\n");
-	fprintf(fp, _("%d equation spaces available in memory,\n%ld kilobytes per equation space.\n"),
-	    N_EQUATIONS, (long) n_tokens * sizeof(token_type) * 2L / 1000L);
-}
-
 /*
  * Display invocation usage info.
  */
@@ -106,8 +91,10 @@ FILE	*fp;
 	fprintf(fp, _("Mathomatic computer algebra system, version %s\n"), VERSION);
 	fprintf(fp, _("Usage: %s [ options ] [ input_files or input ]\n\n"), prog_name);
 	fprintf(fp, _("Options:\n"));
+	fprintf(fp, _("  -a            Enable alternative color mode.\n"));
 	fprintf(fp, _("  -b            Enable bold color mode.\n"));
 	fprintf(fp, _("  -c            Toggle color mode.\n"));
+	fprintf(fp, _("  -d            Set demo mode.\n"));
 	fprintf(fp, _("  -e            Process expressions and commands on the command line.\n"));
 	fprintf(fp, _("  -h            Display this help and exit.\n"));
 	fprintf(fp, _("  -m number     Specify a memory size multiplier.\n"));
@@ -116,11 +103,10 @@ FILE	*fp;
 	fprintf(fp, _("  -s level      Set enforced security level for session.\n"));
 	fprintf(fp, _("  -t            Set test mode.\n"));
 	fprintf(fp, _("  -u            Set unbuffered output with input echo.\n"));
-	fprintf(fp, _("  -v            Display version information, then exit.\n"));
+	fprintf(fp, _("  -v            Display version number, then exit successfully.\n"));
 	fprintf(fp, _("  -w            Wide output mode, sets unlimited width.\n"));
 	fprintf(fp, _("  -x            Enable HTML/XHTML output mode.\n"));
-	fprintf(fp, _("\nPlease refer to the Mathomatic man page for details.\n"));
-	fprintf(fp, _("The man page is viewed by typing \"man mathomatic\" in shell.\n"));
+	fprintf(fp, _("\nPlease refer to the man page for details (type \"man mathomatic\" in shell).\n"));
 }
 
 int
@@ -133,11 +119,11 @@ char	**argv;
 	extern int	optind;
 #endif
 
-	int		i;
+	int		i, rv;
 	char		*cp = NULL;
 	double		numerator, denominator;
 	double		new_size = 0;
-	int		coption = false, boption = false, wide_flag = false;
+	int		aoption = false, coption = false, boption = false, wide_flag = false;
 	int		exit_value = 0;
 
 #if	WIN32_CONSOLE_COLORS
@@ -161,7 +147,7 @@ char	**argv;
 	get_screen_size();
 
 	/* process command line options */
-	while ((i = getopt(argc, argv, "s:bqrtdchuvwxm:e")) >= 0) {
+	while ((i = getopt(argc, argv, "s:abqrtdchuvwxm:e")) >= 0) {
 		switch (i) {
 		case 's':
 #if	SECURE
@@ -178,6 +164,9 @@ char	**argv;
 		case 'w':
 			wide_flag = true;
 			break;
+		case 'a':
+			aoption = true;
+			break;
 		case 'b':
 			boption = true;
 			break;
@@ -185,7 +174,7 @@ char	**argv;
 			coption++;
 			break;
 		case 'x':
-			html_flag = true;
+			html_flag = 1;
 			wide_flag = true;
 			break;
 		case 'm':
@@ -220,7 +209,8 @@ char	**argv;
 			usage(stdout);
 			exit(0);
 		case 'v':
-			version_report();
+/* Don't be fancy, this may be used to test for existence. */
+			printf(_("Mathomatic version %s\n"), VERSION);
 			exit(0);
 		case 'e':
 			eoption = true;
@@ -240,11 +230,11 @@ char	**argv;
 	}
 #if	READLINE
 	if (readline_enabled) {	/* readline_enabled flag must not change after this */
-		if ((cp = getenv("HOME")) && prog_name) {
-#if	CYGWIN || MINGW
-			snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/%s_history", cp, prog_name);
+		if ((cp = getenv("HOME"))) {
+#if	MINGW
+			snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/matho_history", cp);
 #else
-			snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/.%s_history", cp, prog_name);
+			snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/.matho_history", cp);
 #endif
 			history_filename = history_filename_storage;
 		}
@@ -275,18 +265,9 @@ char	**argv;
 	fflush(stdout);
 #if	!SECURE
 	/* read the user options initialization file */
-	if (security_level <= 3 && !test_mode && !demo_mode && !load_rc()) {
-		fprintf(stderr, _("%s: Error loading set options from \"%s\".\n\n"), prog_name, rc_file);
-		snprintf(prompt_str, sizeof(prompt_str), _("Remove faulty set options startup file (y/n)? "));
-		if (get_yes_no()) {
-			if (unlink(rc_file) == 0) {
-				printf(_("File \"%s\" removed.\n"), rc_file);
-				printf(_("Next startup of Mathomatic should be clean with the defaults set.\n"));
-			} else {
-				perror(rc_file);
-			}
-		}
-		printf("\n");
+	if (security_level <= 3 && !test_mode && !demo_mode && !load_rc(true, NULL)) {
+		fprintf(stderr, _("%s: Error loading startup set options from \"%s\".\n"), prog_name, rc_file);
+		fprintf(stderr, _("Use the \"set no save\" command to startup with the program defaults every time.\n\n"));
 	}
 #endif
 	if (wide_flag) {
@@ -297,15 +278,22 @@ char	**argv;
 		color_flag = !color_flag;
 	}
 	if (boption) {
-		color_flag = true;
-		bold_colors = true;
+		color_flag = 1;
+		bold_colors = 1;
+	}
+	if (color_flag && aoption) {
+		color_flag = 2;
 	}
 	if (test_mode) {
-		color_flag = false;
+		color_flag = 0;
 	} else if (!quiet_mode && !eoption) {
 		if (color_flag) {
 #if	WIN32_CONSOLE_COLORS
-			printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "WIN32 CONSOLE", bold_colors ? " bold" : "");
+			if (color_flag == 2) {
+				printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "ANSI", bold_colors ? " bold" : "");
+			} else {
+				printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "WIN32 CONSOLE", bold_colors ? " bold" : "");
+			}
 #else
 			printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "ANSI", bold_colors ? " bold" : "");
 #endif
@@ -325,21 +313,28 @@ char	**argv;
 			printf(_("Operation aborted.\n"));
 			break;
 		}
+		previous_return_value = 0;
 		if (eoption)
 			exit_value = 1;
 	} else {
-		if (!set_signals()) {
-			fprintf(stderr, _("signal(2) setting failed.\n"));
+		if ((rv = set_signals())) {
+			fprintf(stderr, _("C signal handler setting failed!\n"));
+#if	DEBUG
+			fprintf(stderr, "Failing signal is \"%s\".\n", strsignal(rv));
+#endif
+			exit_value = 2;
 		}
 		if (!f_to_fraction(0.5, &numerator, &denominator)
 		    || numerator != 1.0 || denominator != 2.0
 		    || !f_to_fraction(1.0/3.0, &numerator, &denominator)
 		    || numerator != 1.0 || denominator != 3.0) {
-			fprintf(stderr, _("%s: Cannot convert any floating point values to fractions.\n"), prog_name);
+			fprintf(stderr, _("%s: Cannot convert any floating point values to fractions!\n"), prog_name);
 			fprintf(stderr, _("Roots will not work properly.\n"));
+			exit_value = 2;
 		}
 		if (max_memory_usage() <= 0) {
 			fprintf(stderr, _("%s: Calculated maximum memory usage overflows a long integer!\n"), prog_name);
+			exit_value = 2;
 		}
 		if (eoption) {
 			/* process expressions and commands from the command line */
@@ -350,14 +345,23 @@ char	**argv;
 			}
 		} else {
 #if	SECURE
-			if (!html_flag && !quiet_mode && !test_mode)
+			if (!quiet_mode && !test_mode)
 				printf(_("Anything done here is temporary.\n"));
 			if (optind < argc) {
-				warning(_("Command-line argument ignored."));
+				warning(_("File arguments ignored in high security mode."));
 			}
 #else
-			if (!html_flag && !quiet_mode && !test_mode)
-				printf(_("Anything done here is temporary, unless it is saved or redirected.\n"));
+			if (!quiet_mode && !test_mode) {
+				if (optind < argc) {
+					printf(_("Reading in file%s specified on the command line...\n"), (optind == (argc - 1)) ? "" : "s");
+				} else {
+					if (security_level >= 2) {
+						printf(_("Anything done here is temporary.\n"));
+					} else {
+						printf(_("Anything done here is temporary, unless it is saved or redirected.\n"));
+					}
+				}
+			}
 			/* read in files specified on the command line, exit if error */
 			for (i = optind; i < argc && argv[i]; i++) {
 				if (strcmp(argv[i], "-") == 0) {
@@ -386,7 +390,9 @@ main_io_loop(void)
 	char	*cp = NULL;
 
 	for (;;) {
-		default_color();
+		error_str = NULL;
+		warning_str = NULL;
+		default_color(false);
 		snprintf(prompt_str, sizeof(prompt_str), "%d%s", cur_equation + 1, html_flag ? HTML_PROMPT_STR : PROMPT_STR);
 		if ((cp = get_string((char *) tlhs, n_tokens * sizeof(token_type))) == NULL)
 			break;
@@ -396,108 +402,37 @@ main_io_loop(void)
 
 /*
  * All signal(2) initialization goes here.
+ * Attach all necessary C signals to their handler functions.
  *
- * Return true on success.
+ * Return zero on success, or a non-zero unsettable signal number on error.
  */
 int
 set_signals(void)
 {
-	int	rv = true;
+	int	rv = 0;
 
 	if (signal(SIGFPE, fphandler) == SIG_ERR)
-		rv = false;
+		rv = SIGFPE;
 	if (signal(SIGINT, inthandler) == SIG_ERR)
-		rv = false;
+		rv = SIGINT;
 	if (signal(SIGTERM, exithandler) == SIG_ERR)
-		rv = false;
+		rv = SIGTERM;
 #if	!MINGW
 	if (signal(SIGHUP, exithandler) == SIG_ERR)
-		rv = false;
+		rv = SIGHUP;
 #endif
 #if	UNIX || CYGWIN
 	if (signal(SIGWINCH, resizehandler) == SIG_ERR)
-		rv = false;
+		rv = SIGWINCH;
 #endif
 #if	TIMEOUT_SECONDS
 	if (signal(SIGALRM, alarmhandler) == SIG_ERR)
-		rv = false;
+		rv = SIGALRM;
 	alarm(TIMEOUT_SECONDS);
 #endif
 	return rv;
 }
 
-#if	CYGWIN || MINGW
-/*
- * dirname(3) function for Microsoft Windows.
- * dirname(3) strips the non-directory suffix from a filename.
- */
-char *
-dirname_win(cp)
-char	*cp;	/* string containing filename to modify */
-{
-	int	i;
-
-	if (cp == NULL)
-		return(".");
-	i = strlen(cp);
-	while (i >= 0 && cp[i] != '\\' && cp[i] != '/')
-		i--;
-	if (i < 0)
-		return(".");
-	cp[i] = '\0';
-	return(cp);
-}
-#endif
-
-#if	!SECURE
-/*
- * Load set options from startup file "~/.mathomaticrc".
- *
- * Return false if there was an error reading the startup file,
- * otherwise return true.
- */
-int
-load_rc(void)
-{
-	FILE	*fp = NULL;
-	char	buf[MAX_CMD_LEN];
-	char	*cp;
-	int	rv = true;
-
-	cp = getenv("HOME");
-	if (cp) {
-		snprintf(rc_file, sizeof(rc_file), "%s/%s", cp, ".mathomaticrc");
-		fp = fopen(rc_file, "r");
-	}
-#if	CYGWIN || MINGW
-	if (fp == NULL && cp) {
-		snprintf(rc_file, sizeof(rc_file), "%s/%s", cp, "mathomatic.rc");
-		fp = fopen(rc_file, "r");
-	}
-	if (fp == NULL && dir_path) {
-		snprintf(rc_file, sizeof(rc_file), "%s/%s", dir_path, "mathomatic.rc");
-		fp = fopen(rc_file, "r");
-	}
-#endif
-	if (fp == NULL) {
-		return true;
-	}
-	if (!quiet_mode && !eoption) {
-		printf(_("Loading startup set options from \"%s\".\n"), rc_file);
-	}
-	while ((cp = fgets(buf, sizeof(buf), fp)) != NULL) {
-		set_error_level(cp);
-		if (!set_options(cp))
-			rv = false;
-	}
-	if (fclose(fp)) {
-		rv = false;
-		perror(rc_file);
-	}
-	return rv;
-}
-#endif
-
 /*
  * Floating point exception handler.
  * Floating point exceptions are currently ignored.
@@ -525,7 +460,7 @@ int	sig;
 	case 0:
 	case 1:
 		/* wait for graceful abort */
-		printf(_("\nUser interrupt signal received; three times quits Mathomatic.\n"));
+		printf(_("\nUser interrupt signal received; three times in a row quits Mathomatic.\n"));
 		return;
 	case 2:
 		printf(_("\nPress Control-C once more to quit program.\n"));
@@ -590,6 +525,11 @@ int	exit_value;	/* zero if OK, non-zero indicates error return */
 	if (exit_value == 0 && !quiet_mode && !eoption && !html_flag) {
 		printf(_("ByeBye!! from Mathomatic.\n"));
 	}
+#if	VALGRIND
+	printf("Deallocating all Mathomatic allocated memory for valgrind memory leak checking...\n");
+	printf("If you are not using valgrind, please compile without -DVALGRIND.\n");
+        free_mem();     /* Free all known memory buffers to check for memory leaks with something like valgrind(1). */
+#endif
 	exit(exit_value);
 }
 #endif
