@@ -13,7 +13,7 @@ echo '</head>' >>"$htmlfile"
 echo '<body>' >>"$htmlfile"
 echo '<pre>' >>"$htmlfile"
 
-./mathomatic -e "help all main equations constants >>$htmlfile" || (rm "$htmlfile"; exit 1)
+./mathomatic -e "set html all" "help all main equations constants >>$htmlfile" || (rm "$htmlfile"; exit 1)
 
 echo >>"$htmlfile"
 echo '<strong>For more information, visit <a href="http://www.mathomatic.org">www.mathomatic.org</a></strong>' >>"$htmlfile"
