@@ -171,12 +171,13 @@ int		allow_divides;	/* allow variable to be right of a divide (negative exponent
 
 /*
  * Factor polynomials by calling pf_sub() for every additive sub-expression.
- * Factors repeated factor polynomials (like (x+1)^5) if "do_repeat".
- * And factors polynomials with symbolic factors (like (x+a)*(x+b)).
+ * Factors repeated factor polynomials (like (x+1)^5) if do_repeat is true.
+ * And always factors multivariate polynomials with symbolic factors (like (x+a)*(x+b)).
  *
  * Does not factor polynomials with different numeric factors (like (x+1)*(x+2)).
- * Because of floating point round-off error,
- * this routine does not succeed at factoring very large polynomials.
+ *
+ * Because of accumulated floating point round-off error,
+ * this routine does not succeed at factoring most large polynomials.
  *
  * Return true if equation side was modified (factored).
  */
@@ -361,7 +362,7 @@ int		do_repeat;	/* factor repeated factors flag */
 #if	!SILENT
 		if (debug_level >= 3) {
 			list_var(v, 0);
-			fprintf(gfp, _("Differentiation successful using variable (%s).\n"), var_str);
+			fprintf(gfp, _("Differentiation successful using variable %s.\n"), var_str);
 		}
 #endif
 		simp_loop(tlhs, &n_tlhs);
@@ -437,7 +438,9 @@ next_v:
 			if (find_greatest_power(tlhs, n_tlhs, &v, &d, &j, &k, &div_flag) != 1) {
 				continue;
 			}
-			blt(tlhs, &tlhs[j], k * sizeof(token_type));
+			if (j) {
+				blt(tlhs, &tlhs[j], k * sizeof(token_type));
+			}
 			n_tlhs = k;
 #if	!SILENT
 			if (debug_level >= 3) {
@@ -497,6 +500,7 @@ skip_factor:
 			error_bug("Error in result of pf_sub().");
 		}
 #endif
+		debug_string(1, "Resulting factors of pf_sub():");
 		side_debug(1, &equation[loc], i - loc);
 	}
 	return(modified || symbolic_modified);
@@ -860,6 +864,11 @@ int		n;	/* length of expression */
 	int	i;
 	long	v;
 
+#if	DEBUG
+	if (p1 == NULL || n < 1) {
+		error_bug("(p1 == NULL || n < 1) in is_integer_expr().");
+	}
+#endif
 	for (i = 0; i < n; i++) {
 		switch (p1[i].kind) {
 		case OPERATOR:
@@ -1451,7 +1460,7 @@ long		*vp;
 	/* Display debugging info. */
 	if (debug_level >= 3) {
 		list_var(*vp, 0);
-		fprintf(gfp, _("poly_div() starts using base variable (%s):\n"), var_str);
+		fprintf(gfp, _("poly_div() starts using base variable %s:\n"), var_str);
 		side_debug(3, trhs, n_trhs);
 		side_debug(3, tlhs, n_tlhs);
 	}
