@@ -39,6 +39,8 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  * 11/13/11 - Use EXIT_SUCCESS and EXIT_FAILURE macros.
  * 11/30/11 - Added -m option.
  * 12/31/11 - Added a compile-time warning when the number of digits of precision of long doubles is less than 18.
+ * 07/11/12 - Added -v option to display program name with version number and then exit successfully.
+ * 08/09/12 - Allow "matho-primes all" for endless output of consecutive primes.
  */
 
 #include <stdio.h>
@@ -55,7 +57,7 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
 #include <getopt.h>
 #endif
 
-#define	VERSION		"1.3"	/* The current version number of this primes program. */
+#define	VERSION		"1.4"	/* The current version number of this primes program. */
 
 #define	true	1
 #define	false	0
@@ -70,15 +72,19 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
 
 #define	ARR_CNT(a)	((int) (sizeof(a)/sizeof(a[0])))	/* returns the number of elements in an array */
 
+#if	!USE_DOUBLES
 #ifndef	LDBL_DIG
 #define	USE_DOUBLES	1
+#warning "long doubles not supported by this C compiler."
 #elif LDBL_DIG < 18
-#warning "long double number of digits precision is less than 18."
+#warning "long double number of digits precision is less than 18, probably meaning matho-primes is double precision only."
+#warning "Due to the above floating point warning, more testing or compiling with -DUSE_DOUBLES is recommended."
+#endif
 #endif
 
 #if	USE_DOUBLES
 typedef double double_type;
-#warning "Using only double floats instead of long double floats."
+#warning "Using only double floats instead of long double floats, as requested."
 #else
 typedef long double double_type;
 #endif
@@ -163,7 +169,7 @@ do_again:
 	number = 0;
 	count_requested = false;
 /* process command line options: */
-	while ((i = getopt(argc, argv, "c:thup:m:")) >= 0) {
+	while ((i = getopt(argc, argv, "c:thuvp:m:")) >= 0) {
 		switch (i) {
 		case 'c':
 			count_requested = true;
@@ -197,15 +203,26 @@ do_again:
 			setbuf(stdout, NULL);	/* make output unbuffered */
 			setbuf(stderr, NULL);
 			break;
+		case 'v':
+			printf("%s version %s\n", prog_name, VERSION);
+			exit(0);
 		default:
 			usage2(EXIT_FAILURE);
 		}
 	}
-	if (argc > optind) {
-		if (strncmp(argv[optind], "twin", 4) == 0) {
-			twin_flag = true;
-			optind++;
+	for (; argc > optind; optind++) {
+		if (strcasecmp(argv[optind], "all") == 0) {
+			if (start_value < 0)
+				start_value = 0;
+			count_requested = true;
+			number = max_integer;
+			continue;
 		}
+		if (strcasecmp(argv[optind], "twin") == 0) {
+			twin_flag = true;
+			continue;
+		}
+		break;
 	}
 	if (argc > optind && isdigit(argv[optind][0])) {
 		if (get_double_type_int(argv[optind], &start_value)) {
@@ -227,14 +244,22 @@ do_again:
 			}
 		}
 	}
-	if (argc > optind) {
-		if (strncmp(argv[optind], "twin", 4) == 0) {
-			twin_flag = true;
-			optind++;
+	for (; argc > optind; optind++) {
+		if (strcasecmp(argv[optind], "all") == 0) {
+			if (start_value < 0)
+				start_value = 0;
+			count_requested = true;
+			number = max_integer;
+			continue;
 		}
+		if (strcasecmp(argv[optind], "twin") == 0) {
+			twin_flag = true;
+			continue;
+		}
+		break;
 	}
 	if (argc > optind) {
-		if (strncmp(argv[optind], "pal", 3) == 0) {
+		if (strncasecmp(argv[optind], "pal", 3) == 0) {
 			pal_flag = true;
 			optind++;
 		} else {
@@ -248,11 +273,19 @@ do_again:
 			optind++;
 		}
 	}
-	if (argc > optind) {
-		if (strncmp(argv[optind], "twin", 4) == 0) {
-			twin_flag = true;
-			optind++;
+	for (; argc > optind; optind++) {
+		if (strcasecmp(argv[optind], "all") == 0) {
+			if (start_value < 0)
+				start_value = 0;
+			count_requested = true;
+			number = max_integer;
+			continue;
 		}
+		if (strcasecmp(argv[optind], "twin") == 0) {
+			twin_flag = true;
+			continue;
+		}
+		break;
 	}
 	if (argc > optind) {
 		fprintf(stderr, "Unrecognized argument: \"%s\".\n", argv[optind]);
@@ -282,7 +315,7 @@ get1:
 	if (number == 0) {
 get2:
 		fprintf(stderr, "Enter number of%s%s primes to output (0 to end)",
-		    pal_flag ? " palindromic" : "", twin_flag ? " twin" : "");
+		    pal_flag ? " palindromic" : " consecutive", twin_flag ? " twin" : "");
 #if	USE_DOUBLES
 		fprintf(stderr, " (%g): ", default_number);
 #else
@@ -526,8 +559,8 @@ test_pal(double_type d, double_type base)
 void
 usage(int ev)
 {
-	printf("\nPrime number generator version %s\n", VERSION);
-	printf("Usage: %s [start [stop]] [\"twin\"] [\"pal\" [base]]\n\n", prog_name);
+	printf("Prime number generator version %s\n", VERSION);
+	printf("Usage: %s [start [stop] or \"all\"] [\"twin\"] [\"pal\" [base]]\n\n", prog_name);
 #if	USE_DOUBLES
 	printf("Generate consecutive prime numbers from start to stop, up to %g.\n", max_integer);
 #else
@@ -542,7 +575,7 @@ usage(int ev)
 void
 usage2(int ev)
 {
-	printf("\nPrime number generator version %s\n", VERSION);
+	printf("Prime number generator version %s\n", VERSION);
 	printf("Usage: %s [options] [start [stop]]\n\n", prog_name);
 #if	USE_DOUBLES
 	printf("Generate consecutive prime numbers from start to stop, up to %g.\n", max_integer);
@@ -556,5 +589,6 @@ usage2(int ev)
 	printf("  -p base          Output only palindromic primes.\n");
 	printf("  -t               Output only twin primes.\n");
 	printf("  -u               Set all output to be unbuffered.\n");
+	printf("  -v               Display version number, then exit successfully.\n");
 	exit(ev);
 }
