@@ -3,7 +3,9 @@
 /* This file is required to compile Mathomatic quietly with the -Wall compiler option. */
 
 /* am.c */
+void display_startup_message(FILE *fp);
 void error(const char *str);
+void reset_error(void);
 void warning(const char *str);
 void error_huge(void);
 void error_bug(const char *str);
@@ -57,10 +59,12 @@ int exp_contains_nan(token_type *p1, int n1);
 int exp_is_numeric(token_type *p1, int n1);
 int exp_is_absolute(token_type *p1, int n1);
 int check_divide_by_zero(double denominator);
+int load_rc(int return_true_if_no_file, FILE *ofp);
 /* cmds.c */
 int plot_cmd(char *cp);
 int version_cmd(char *cp);
 long max_memory_usage(void);
+int show_status(FILE *ofp);
 int version_report(void);
 int solve_cmd(char *cp);
 int sum_cmd(char *cp);
@@ -71,10 +75,10 @@ int push_cmd(char *cp);
 int push_en(int en);
 int output_current_directory(FILE *ofp);
 int fprintf_escaped(FILE *ofp, char *cp);
-void output_options(FILE *ofp);
+void output_options(FILE *ofp, int all_set_options);
 int skip_no(char **cpp);
-int save_set_options(void);
-int set_options(char *cp);
+int save_set_options(char *cp);
+int set_options(char *cp, int loading_startup_file);
 int set_cmd(char *cp);
 int echo_cmd(char *cp);
 int pause_cmd(char *cp);
@@ -97,13 +101,14 @@ int approximate_cmd(char *cp);
 int replace_cmd(char *cp);
 int simplify_cmd(char *cp);
 int factor_cmd(char *cp);
+int display_term_count(int en);
 int unfactor_cmd(char *cp);
 int div_loc_find(token_type *expression, int n);
 int fraction_cmd(char *cp);
 int quit_cmd(char *cp);
 int read_cmd(char *cp);
 int read_file(char *cp);
-int read_sub(FILE *fp);
+int read_sub(FILE *fp, char *filename);
 int edit_cmd(char *cp);
 int save_cmd(char *cp);
 /* complex.c */
@@ -137,10 +142,10 @@ int factor_power(token_type *equation, int *np);
 /* factor_int.c */
 int factor_one(double value);
 double multiply_out_unique(void);
-void display_unique(void);
+int display_unique(void);
 int is_prime(void);
 int factor_int(token_type *equation, int *np);
-void factor_int_equation(int n);
+int factor_int_equation(int n);
 int list_factor(token_type *equation, int *np, int factor_flag);
 int factor_constants(token_type *equation, int *np, int level_code);
 /* gcd.c */
@@ -175,7 +180,7 @@ int nintegrate_cmd(char *cp);
 /* list.c */
 void reset_attr(void);
 int set_color(int color);
-void default_color(void);
+void default_color(int set_no_color_flag);
 int display_all_colors(void);
 int list1_sub(int n, int export_flag);
 int list_sub(int n);
@@ -194,12 +199,10 @@ int list_code(token_type *equation, int *np, int outflag, char *string, enum lan
 char *flist_equation_string(int n);
 int flist_equation(int n);
 /* main.c */
-void display_startup_message(FILE *fp);
 void usage(FILE *fp);
 int main(int argc, char **argv);
 void main_io_loop(void);
 int set_signals(void);
-int load_rc(void);
 void fphandler(int sig);
 void inthandler(int sig);
 void alarmhandler(int sig);
@@ -255,7 +258,7 @@ void simpv_equation(int n, long v);
 int factor_imaginary(token_type *equation, int *np);
 void factorv(token_type *equation, int *np, long v);
 void calc_simp(token_type *equation, int *np);
-int approximate(token_type *equation, int *np);
+void approximate(token_type *equation, int *np);
 int simp_i(token_type *equation, int *np);
 void simp_divide(token_type *equation, int *np);
 void simp2_divide(token_type *equation, int *np, long v, int fc_level);
