@@ -37,9 +37,11 @@ static int	prev_n1, prev_n2;
 static int	last_int_var = 0;
 
 /*
- * Solve using equation spaces.
+ * Solve using equation spaces.  Almost always displays a message.
  *
- * Return true if successful.
+ * Return true if successful.  If successful, you should display the solve result.
+ * You are allowed to simplify the result before display however,
+ * preferably with "simplify quick".  Just plain "simplify" expands too much sometimes.
  */
 int
 solve_espace(want, have)
@@ -48,8 +50,17 @@ int	have;	/* equation number to solve */
 {
 	int	i;
 	jmp_buf	save_save;
-	int	rv = 0;	/* return value */
+	int	rv = 0;		/* solve_sub() return value */
 
+	if (want == have || !equation_space_is_equation(have)) {
+#if	LIBRARY || !HELP
+		error(_("Solving requires an equation."));
+#else
+		error(_("Please enter an equation to solve, or a command like \"help\"."));
+#endif
+		printf(_("Solve failed for equation space #%d.\n"), have + 1);
+		return false;
+	}
 	blt(save_save, jmp_save, sizeof(jmp_save));
 	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
 		clean_up();
@@ -75,6 +86,8 @@ int	have;	/* equation number to solve */
 	blt(jmp_save, save_save, sizeof(jmp_save));
 	if (rv <= 0) {
 		printf(_("Solve failed for equation space #%d.\n"), have + 1);
+	} else {
+		debug_string(0, _("Solve successful:"));
 	}
 	return(rv > 0);
 }
@@ -98,7 +111,7 @@ int	have;	/* equation number to solve */
 int
 solve_sub(wantp, wantn, leftp, leftnp, rightp, rightnp)
 token_type	*wantp;		/* expression to solve for */
-int		wantn;		/* length of expression to solve for (currently should always be 1) */
+int		wantn;		/* length of expression to solve for */
 token_type	*leftp;		/* LHS of equation */
 int		*leftnp;	/* pointer to length of LHS */
 token_type	*rightp;	/* RHS of equation */
@@ -128,7 +141,7 @@ int		*rightnp;	/* pointer to length of RHS */
 #if	LIBRARY || !HELP
 		error(_("Solving requires an equation."));
 #else
-		error(_("Please enter an equation or a command like \"help\"."));
+		error(_("Please enter an equation to solve, or a command like \"help\"."));
 #endif
 		return false;
 	}
@@ -283,15 +296,16 @@ see_work:
 	/* See if we have solved the equation. */
 	if (se_compare(wantp, wantn, tlhs, n_tlhs, &diff_sign) && !diff_sign) {
 		if (zsolve) {
+			debug_string(1, "Simplifying the zero solve until there are no more divides:");
 zero_simp:
-			list_tdebug(4);
+			list_tdebug(2);
 			uf_power(trhs, &n_trhs);
 			do {
 				do {
-					simp_ssub(trhs, &n_trhs, 0L, 0.0, false, true, 4 /* was 0 */);
+					simp_ssub(trhs, &n_trhs, 0L, 0.0, false, true, 4);
 				} while (uf_power(trhs, &n_trhs));
 			} while (super_factor(trhs, &n_trhs, 1));
-			list_tdebug(3);
+			list_tdebug(1);
 			ep = &trhs[n_trhs];
 			op = 0;
 			for (p1 = trhs + 1; p1 < ep; p1 += 2) {
@@ -324,15 +338,35 @@ zero_simp:
 					p1 = b1;
 					for (p1++; p1 < ep && p1->level > 1; p1 += 2)
 						;
+#if	DEBUG
+					if (p1 != ep && (p1->kind != OPERATOR || p1->token.operatr != TIMES)) {
+						error_bug("Operator mix up in zero_simp.");
+					}
+#endif
+					if ((p1 - 2) > b1) {
+						p1 -= 2;
+						if (p1->token.operatr == POWER && p1->level == 2
+						    /* && ((p1 - 2) <= b1 || (p1 - 2)->token.operatr != POWER || (p1 - 2)->level != 3) */) {
+							p1++;
+							if (p1->level == 2 && p1->kind == CONSTANT && p1->token.constant > 0.0) {
+								p1->token.constant = 1.0;
+								goto zero_simp;
+							}
+							p1++;
+						} else
+							p1 += 2;
+					}
 					p1++;
 				}
 				break;
 			case POWER:
-				p1++;
-				if (p1->level == 1 && p1->kind == CONSTANT && p1->token.constant > 0.0) {
-					n_trhs -= 2;
-					goto zero_simp;
-				}
+/*				if ((p1 - 2) <= trhs || (p1 - 2)->token.operatr != POWER || (p1 - 2)->level != 2) { */
+					p1++;
+					if (p1->level == 1 && p1->kind == CONSTANT && p1->token.constant > 0.0) {
+						n_trhs -= 2;
+						goto zero_simp;
+					}
+/*				} */
 				break;
 			}
 			debug_string(1, _("Solve for zero completed:"));
@@ -478,7 +512,6 @@ zero_simp:
 				}
 				if (uf_flag) {
 					simps_side(tlhs, &n_tlhs, zsolve);
-/*					list_tdebug(1); */
 					uf_flag = false;
 					goto see_work;
 				}
@@ -638,10 +671,10 @@ long	v;
 	partial_flag = false;
 	ufactor(tlhs, &n_tlhs);
 	partial_flag = true;
-	symb_flag = symblify;
+/*	symb_flag = symblify; */
 	simp_ssub(tlhs, &n_tlhs, v, d, true, false, 2);
 	simp_ssub(tlhs, &n_tlhs, 0L, 1.0, true, true, 2);
-	symb_flag = false;
+/*	symb_flag = false; */
 	list_tdebug(1);
 
 isolate:
@@ -860,7 +893,7 @@ long	v;	/* solve variable */
 #if	!SILENT
 	if (debug_level >= 0) {
 		list_var(v, 0);
-		fprintf(gfp, _("Equation is a degree %.*g polynomial equation in (%s).\n"), precision, high_power, var_str);
+		fprintf(gfp, _("Equation is a degree %.*g polynomial equation in %s.\n"), precision, high_power, var_str);
 	}
 #endif
 	if (a1p > trhs && (a1p - 1)->token.operatr == MINUS)
@@ -1159,6 +1192,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		debug_string(1, _("Taking logarithm of both equation sides:"));
 		errno = 0;
 		c1 = complex_div(complex_log(c1), complex_log(c2));
+		check_err();
 		*side2np = 0;
 		side2p[*side2np].level = 1;
 		side2p[*side2np].kind = CONSTANT;
@@ -1183,7 +1217,6 @@ int		*side2np;	/* pointer to the length of "side2p" */
 
 		blt(side1p, p1 + 1, (*side1np - (operandn + 1)) * sizeof(token_type));
 		*side1np -= operandn + 1;
-		check_err();
 		return true;
 	}
 	if (op == MODULUS) {
